package footbalmanager;

import java.io.*;

/**
 * Trida pro ukladani a nacitani dat do souboru.
 * Implementuje rozhrani IStorage.
 *
 * @author Jakub Trmota
 */
public class FileStorage implements IStorage {

    /**
     * Debug mod, vysisuje chybove zpravy
     */
    private boolean debug = false;
    /**
     * Soubor s tymy.
     */
    private final String FILE_TEAMS = "teams.dat";
    /**
     * Soubor se zapasy.
     */
    private final String FILE_MATCHES = "matches.dat";
    /**
     * Soubor s ligou.
     */
    private final String FILE_LEAGUE = "league.dat";
    /**
     * Singleton - ulozena instance.
     */
    private static FileStorage instance = null;

    /**
     * Konstruktor.
     *
     * @param debug debugovaci mod
     */
    public FileStorage(boolean debug) {
        this.debug = debug;
    }

    /**
     * Vrati instanci objektu - singleton.
     *
     * @param debug debugovaci mod
     * @return instance objektu
     */
    public static FileStorage getStorage(boolean debug) {
        if (instance == null) {
            instance = new FileStorage(debug);
        }

        return instance;
    }

    /**
     * Nacte tymy.
     *
     * @param teams pole s tymy
     * @return true, pokud se podari nacist
     */
    public boolean loadTeams(Team[] teams) {
        try {
            BufferedReader input = new BufferedReader(new FileReader(this.FILE_TEAMS));

            try {
                for (int i = 0; i < 16; i++) {
                    String name = input.readLine();
                    int ofence = new Integer(input.readLine()).intValue();
                    int defence = new Integer(input.readLine()).intValue();
                    teams[i] = new Team(name, ofence, defence);
                }
            } finally {
                input.close();
            }
        } catch (IOException e) {
            if (debug) {
                e.printStackTrace();
            }

            return false;
        }

        return true;
    }

    /**
     * Nacte zapasy.
     *
     * @param matches pole se zapasy
     * @return true, pokud se podari nacist
     */
    public boolean loadMatches(Match[][] matches) {
        try {
            BufferedReader input = new BufferedReader(new FileReader(this.FILE_MATCHES));

            try {
                for (int i = 0; i < 15; i++) {
                    for (int x = 0; x < 8; x++) {
                        int home = new Integer(input.readLine()).intValue();
                        int visitor = new Integer(input.readLine()).intValue();
                        matches[i][x] = new Match(home - 1, visitor - 1);
                        matches[i + 15][x] = new Match(visitor - 1, home - 1);
                    }
                }
            } finally {
                input.close();
            }
        } catch (IOException e) {
            if (this.debug) {
                e.printStackTrace();
            }

            return false;
        }

        return true;
    }

    /**
     * Nacte ligu ze souboru.
     *
     * @param teams pole tymu
     * @param matches pole zapasu
     * @return true, pokud se poradi nacist
     */
    public boolean loadLeague(Team[] teams, Match[][] matches) {
        try {
            ObjectInputStream input = new ObjectInputStream(new FileInputStream(this.FILE_LEAGUE));

            try {
                // Save teams
                for (int i = 0; i < 16; i++) {
                    teams[i] = (Team) input.readObject();
                }

                // Save matches
                for (int i = 0; i < 30; i++) {
                    for (int x = 0; x < 8; x++) {
                        matches[i][x] = (Match) input.readObject();
                    }
                }
            } finally {
                input.close();
            }
        } catch (Exception e) {
            if (this.debug) {
                e.printStackTrace();
            }

            return false;
        }

        return true;
    }

    /**
     * Ulozi ligu do souboru.
     *
     * @param teams pole s tymy
     * @param matches pole se zapasy
     * @return true, pokud se podari ulozit
     */
    public boolean saveLeague(Team[] teams, Match[][] matches) {
        try {
            ObjectOutputStream output = new ObjectOutputStream(new FileOutputStream(this.FILE_LEAGUE));

            try {
                // Save teams
                for (int i = 0; i < 16; i++) {
                    output.writeObject(teams[i]);
                }

                // Save matches
                for (int i = 0; i < 30; i++) {
                    for (int x = 0; x < 8; x++) {
                        output.writeObject(matches[i][x]);
                    }
                }
            } finally {
                output.close();
            }
        } catch (Exception e) {
            if (this.debug) {
                e.printStackTrace();
            }

            return false;
        }

        return true;
    }
}
